function disp_img = calcDisparity(...
    left_img, right_img, patch_size, min_disp, max_disp)
% TODO: compute disparity image between left and right image 
% conventions: 
% - return a disparity image with the same size like the input images
% - set disparity to 0 if the patch comparison is undefined
% - patches are of size patch_size x patch_size and centered at the pixel so
% patch_size should be odd
% - keep disparities within [min_disp, max_disp] interval

disparity = zeros(size(left_img), 'single');

% block size
half_patch_size = (patch_size - 1) / 2;

% image size
[imgHeight, imgWidth] = size(left_img);

for m = 1 : imgHeight
    	
	% row boundries 
	minr = max(1, m - half_patch_size);
    maxr = min(imgHeight, m + half_patch_size);
	
    for n = 1 : imgWidth
        
		% col boundries
		minc = max(1, n - half_patch_size);
        maxc = min(imgWidth, n + half_patch_size);
        
		% Define the search boundaries as offsets from the template location.
		% Limit the search so that we don't go outside of the image. 
		% 'mind' is the the maximum number of pixels we can search to the left.
		% 'maxd' is the maximum number of pixels we can search to the right.
		mind = max(-min_disp, 1 - minc); 
        maxd = min(max_disp, imgWidth - maxc);

		% Select the block from the right image to use as the template.
        template = right_img(minr:maxr, minc:maxc);
		
		% Get the number of blocks in this search.
		numBlocks = maxd - mind + 1;
		
		% Create a vector to hold the block differences.
		blockDiffs = zeros(numBlocks, 1);
		
		% Calculate the difference between the template and each of the blocks.
		for (i = mind : maxd)
		
			% Select the block from the left image at the distance 'i'.
			block = left_img(minr:maxr, (minc + i):(maxc + i));
		
			% Compute the 1-based index of this block into the 'blockDiffs' vector.
			blockIndex = i - mind + 1;
		
			% SAD
			blockDiffs(blockIndex, 1) = sum(sum(abs(template - block)));
            
            % and SSD
            %blockDiffs(blockIndex, 1) = sum(sum(double(template - block)*double(template - block)));
            %blockDiffs(blockIndex, 1) = sum(sum((template - block).^2));
		end
		
		% Sort the SAD values to find the closest match).
		[~, sortedIndeces] = sort(blockDiffs);
        
		% Get the 1-based index of the closest-matching block.
		bestMatchIndex = sortedIndeces(1, 1);
        
        % Skip if second best match is way too similar
        secondbestMatchIndex = sortedIndeces(2, 1);
        if ( abs(blockDiffs(bestMatchIndex) - blockDiffs(secondbestMatchIndex)) <= 2)
            disparity(m, n) = 0;
            continue;
        end
		
		% Convert the 1-based index of this block back into an offset.
		% This is the final disparity value produced by basic block matching.
		d = bestMatchIndex + mind - 1;
			
		% Calculate a sub-pixel estimate of the disparity by interpolating.
		% Sub-pixel estimation requires a block to the left and right, so we 
		% skip it if the best matching block is at either edge of the search
		% window.
		if ((bestMatchIndex == 1) || (bestMatchIndex == numBlocks))
			% Skip sub-pixel estimation and store the initial disparity value.
			disparity(m, n) = d;
		else
			% Grab the SAD values at the closest matching block (C2) and it's 
			% immediate neighbors (C1 and C3).
			C1 = blockDiffs(bestMatchIndex - 1);
			C2 = blockDiffs(bestMatchIndex);
			C3 = blockDiffs(bestMatchIndex + 1);
			
			% Adjust the disparity by some fraction.
			% We're estimating the subpixel location of the true best match.
			disparity(m, n) = d - (0.5 * (C3 - C1) / (C1 - (2*C2) + C3));
		end
    end

	% Update progress every 10th row.
	if (mod(m, 10) == 0)
		fprintf('  Image row %d / %d (%.0f%%)\n', m, imgHeight, (m / imgHeight) * 100);
    end
    
   
		
end

  disp_img = disparity;

end

